// Very simple and inaccurate, but it's better than nothing.
float water_fog() {
    vec2 ScreenPos = gl_FragCoord.xy / resolution;
    float TerrainDepth = texture2D(depthtex1, ScreenPos).x;
    TerrainDepth = linearize_depth(TerrainDepth);
    float ScreenDepth = linearize_depth(gl_FragCoord.z);
    return smoothstep(0, 64, TerrainDepth - ScreenDepth) * WATER_FOG_STRENGTH;
}

float schlick(vec3 V, vec3 N) {
    const float R = 0.1;
    float Theta = clamp(1 - dot(-V, N), 0, 1);
    Theta = max(Theta, 0); // https://www.youtube.com/watch?v=BcxKINWMD8M
    return R + (1 - R) * (pow(Theta, 5.0));
}

vec3 get_water_normal(vec2 Coords) {
    vec2 N = noise_water(Coords);
    return normalize(vec3(N.x, N.y, 1 - (N.x * N.x + N.y * N.y)));
}

vec3 sky_reflection(vec3 ReflectedVec, float Dist) {
    #ifdef DIMENSION_OVERWORLD
    vec3 SunGlare = get_sun_glare(Dist);
    vec3 Reflection = get_sky(ReflectedVec, SunGlare);
    Reflection.rgb += round_sun(Dist) * 4 * is_indoors_smooth();
    return Reflection;
    #else
    return fogColor.rgb;
    #endif
}

vec3 ssr(vec3 RVec, float Dist, vec3 ViewPos, float Fresnel) {
    vec3 ScreenPos = vec3(gl_FragCoord.xy / resolution, gl_FragCoord.z);
    // Convert RVec to screenspace
    vec3 Offset = normalize(view_screen(ViewPos + RVec) - ScreenPos);
    vec3 Len = (step(0, Offset) - ScreenPos) / Offset;
    float MinLen = min(Len.x, min(Len.y, Len.z)) / SSR_STEPS;
    Offset *= MinLen;

    float Noise = dither(gl_FragCoord.xy);
    vec3 ExpectedPos = ScreenPos + Offset * Noise;
    for (int i = 1; i <= SSR_STEPS; i++) {
        float RealDepth = get_depth_solid(ExpectedPos.xy);
        if (RealDepth < 0.56) {
            break;
        }

        if (ExpectedPos.z > RealDepth) {
            // Depth based rejection
            if (ExpectedPos.z - RealDepth > Offset.z * 2) break;

            // Binary refinement
            for (int j = 1; j <= int(round(Fresnel * 3)); j++) {
                Offset /= 2;
                vec3 EPos1 = ExpectedPos - Offset;
                float RDepth1 = get_depth_solid(EPos1.xy);
                if (EPos1.z > RDepth1) {
                    ExpectedPos = EPos1;
                }
            }
            return texture2D(gaux1, ExpectedPos.xy).rgb;
        }
        ExpectedPos += Offset;
    }
    return sky_reflection(RVec, Dist);
}

vec4 get_fancy_water(vec3 ViewPos, vec4 BaseColor, float SkyBrightness) {
    #ifndef DISTANT_HORIZONS
    if (isEyeInWater == 0) {
        BaseColor.a = min(BaseColor.a + water_fog(), 1);
    }
    #endif
    vec3 ViewPosN = normalize(ViewPos);
    vec3 WorldPos = to_player_pos(ViewPos) + cameraPosition;

    vec3 NormalMap = get_water_normal(WorldPos.xz);
    vec3 WaterNormal = TBN * NormalMap;

    vec3 ReflectedVec = reflect(ViewPosN, WaterNormal);
    float Dist = distance(ReflectedVec, sunPosN);
    float Fresnel = schlick(ViewPosN, WaterNormal) * SkyBrightness;

    #ifdef SSR
    vec3 Reflection = ssr(ReflectedVec, Dist, ViewPos, Fresnel);
    #else
    vec3 Reflection = sky_reflection(ReflectedVec, Dist);
    #endif

    BaseColor.rgb = mix(BaseColor.rgb, Reflection, Fresnel);

    vec3 SkyColor = get_sky(ViewPosN, get_sun_glare(distance(ViewPosN, sunPosN)));
    BaseColor.rgb = get_fog_main(WorldPos - cameraPosition, BaseColor.rgb, gl_FragCoord.z, SkyColor);
    return BaseColor;
}
