#include "/lib/all_the_libs.glsl"

#include "/global/sky.glsl"
#include "/global/fog.glsl"
#include "/global/post/ssao.glsl"
#include "/global/post/godrays.glsl"

// Sky, Clouds, etc

varying vec2 texcoord;

#if defined SSR && TAA_MODE == 0
/* DRAWBUFFERS:04 */
#else
/* DRAWBUFFERS:0 */
#endif

void main() {
    vec4 Color = texture2D(colortex0, texcoord);
    float Depth = get_depth(texcoord);
    vec3 ViewPos;
    #ifdef DISTANT_HORIZONS
    // Need to use dhProjectionInverse but only on DH terrain. Can't use #define DH_TERRAIN in this file
    if (texture2D(depthtex0, texcoord).x >= 1 && Depth < 1)
        ViewPos = to_view_pos_dh(vec3(texcoord, Depth));
    else
        #endif
        ViewPos = to_view_pos(vec3(texcoord, Depth));

    vec3 PlayerPos = mat3(gbufferModelViewInverse) * ViewPos;

    vec3 ViewPosN = normalize(ViewPos);
    vec3 PlayerPosN = normalize(PlayerPos);

    float Dist = distance(ViewPosN, sunPosN);
    vec3 SunGlare = get_sun_glare(Dist);
    vec3 SkyColor = get_sky_main(ViewPosN, PlayerPosN, SunGlare);

    if (Depth >= 1) {
        #ifndef DIMENSION_OVERWORLD
        Color = vec4(0, 0, 0, 1);
        #elif defined ROUND_SUN
        SkyColor += round_sun(Dist);
        #endif

        #ifdef DEBUG_OVERRIDE_SKY
        Color.rgb = SkyColor;
        #else
        Color.rgb += SkyColor;
        #endif

        if (PlayerPos.y > 0) {
            #if defined DIMENSION_OVERWORLD || defined DIMENSION_END
            Color.rgb += get_stars(PlayerPos);
            #endif

            #if defined DIMENSION_OVERWORLD && defined FANCY_CLOUDS
            Color.rgb = get_cloud_noise(PlayerPos, PlayerPosN, SunGlare, Color.rgb);
            #endif
        }
    }
    #ifdef SSAO
    else if (Depth >= 0.56) { // Skip hand & sky
        vec3 Normal = decodeUnitVector(texture2D(colortex1, texcoord).rg * 2 - 1);
        Color.rgb = ssao(Color.rgb, Normal, ViewPos);
    }
    #endif

    if (Depth >= 0.56) { // Skip JUST the hand
        Color.rgb = get_fog_main(PlayerPos, Color.rgb, Depth, SkyColor);

        #if defined DIMENSION_OVERWORLD && defined GODRAYS
        Color.rgb += godrays(vec3(texcoord, Depth));
        #endif
    }

    #if defined SSR && TAA_MODE == 0
    gl_FragData[1] = Color;
    #endif

    gl_FragData[0] = Color;
}
