#define DH_TERRAIN

#include "/lib/all_the_libs.glsl"
uniform sampler2D lightmap;
uniform sampler2D gtexture;

varying vec2 texcoord;
varying vec4 glcolor;
varying vec2 LightmapCoords;

#include "/global/lighting.fsh"
#include "/global/sky.glsl"
#include "/global/fog.glsl"

varying mat3 TBN;
#include "/global/water.glsl"

vec4 get_translucent_basic(vec3 TweakedLM, vec3 ViewPosN, vec3 PlayerPos) {
    vec4 Color = glcolor;
    Color.rgb = to_linear(Color.rgb);
    if (Color.a < 0.1) {
        discard;
    }
    Color.rgb *= TweakedLM;
    vec3 SkyColor = get_sky_main(ViewPosN, normalize(PlayerPos), get_sun_glare(distance(ViewPosN, sunPosN)));
    Color.rgb = get_fog_main(PlayerPos, Color.rgb, gl_FragCoord.z, SkyColor);

    return Color;
}

/* DRAWBUFFERS:0 */

void main() {

    vec3 ScreenPos = vec3(gl_FragCoord.xy/resolution, gl_FragCoord.z);
    vec3 ViewPos = to_view_pos(ScreenPos);
    vec3 ViewPosN = normalize(ViewPos);
    vec3 PlayerPos = to_player_pos(ViewPos);

    if(max(length(PlayerPos.xz), abs(PlayerPos.y) * 2.0) < far-DH_CUTOFF) {
        discard; return;
    }

    float Depth = texture2D(depthtex0, ScreenPos.xy).x;

    if(Depth < 1) {
        discard; return;
    }

    vec3 TweakedLM = tweak_lightmap();
    vec4 Color;
    #ifndef FANCY_WATER
    Color = get_translucent_basic(TweakedLM, ViewPosN, PlayerPos);
    #else
    if(material == 10002) {

        vec4 BaseColor = vec4(WATER_RED, WATER_GREEN, WATER_BLUE, WATER_ALPHA);
        BaseColor.rgb = to_linear(BaseColor.rgb);
        BaseColor.rgb *= TweakedLM;
        Color = get_fancy_water(ViewPos, BaseColor, clamp(LightmapCoords.y, 0, 1));
    }
    else {
        Color = get_translucent_basic(TweakedLM, ViewPosN, PlayerPos);
    }
    #endif

    gl_FragData[0] = Color;

}
